/********************************************************************

	filename:   netstream.h
	author:     Mikael Swartling
	e-mail:     e98msv@du.se

	description:
	------------
	Network streams is a set of easy-to-use classes to deal with peer-to-peer
	communication over the Internet. It uses << and >> operators to stream
	pre/user defined datatypes, and commands for sending/receiving arbitrary
	buffers of data over a connection. It operates on a very low level, making
	it independent of any application-level protocol.

	-----

	This software is provided 'as-is', without any express or implied warranty.
	In no event will the author be held liable for any damages arising from the
	use of this software.

	Permission is hereby granted to use, copy, modify, and distribute this
	source code, or portions hereof, for any purpose, without fee, subject to
	the following restrictions:
	
	1. The origin of this source code must not be misrepresented. You must not
	   claim that you wrote the original software. If you use this software in
	   a product, an acknowledgment in the product documentation would be
	   appreciated but is not required.

	2. Altered versions must be plainly marked as such and must not be
	   misrepresented as being the original source.
		
	3. This notice must not be removed or altered from any source distribution.

*********************************************************************/

#ifndef __NETSTREAM_H__
#define __NETSTREAM_H__

// --------------------------------------------------------------------------------
// Add the WinSock library
// --------------------------------------------------------------------------------
#ifdef _WIN32
#include <winsock2.h>
#pragma comment(lib, "ws2_32.lib")
#endif


// --------------------------------------------------------------------------------
// 
// --------------------------------------------------------------------------------
#include <string>

typedef std::string string;

// --------------------------------------------------------------------------------
// On windows we need to setup WinSock before using sockets.
// --------------------------------------------------------------------------------
#ifdef _WIN32
bool WinsockStartup();
bool WinsockCleanup();
#endif

// --------------------------------------------------------------------------------
// netsocket:
// It provides some functions that is common to both netstream and netserver.
// --------------------------------------------------------------------------------
class netsocket
{
public:

	netsocket();
	virtual ~netsocket();

	bool   close();
	bool   isopen() const;
	string gethostname() const;

protected:

	bool create();

protected:

	SOCKET m_socket;
};

// --------------------------------------------------------------------------------
// netsream:
// Stream data from/to another computer.
// --------------------------------------------------------------------------------
class netstream : public netsocket
{
public:
	netstream() : netsocket() {}
	~netstream();

	bool attach(SOCKET socket);

	bool connect(const string &hostName, int hostPort);
	bool connect(const string &hostNameAndPort);

	bool send(const void *data, int len) const;
	int  recv(      void *data, int len) const;

	int  canrecv();

	string getpeername() const;

	template<typename T> netstream &operator <<(const T &data) {send(&data, sizeof T);return *this;}
	template<typename T> netstream &operator >>(      T &data) {recv(&data, sizeof T);return *this;}

	netstream &operator <<(const string &data);
	netstream &operator >>(      string &data);

	netstream &operator <<(const char   *data);
};

// --------------------------------------------------------------------------------
// netserver:
// The class for dealing with incomming connections.
// --------------------------------------------------------------------------------
class netserver : public netsocket
{
public:
	netserver() : netsocket() {}
	~netserver();

	bool listen(int port, const string &adapter = string(), int backlog = 5);
	bool accept(netstream &socket) const;

	bool canaccept() const;
};

#endif // __NETSTREAM_H__
